// This source code is a part of Project Violet.
// Copyright (C) 2021.violet-team. Licensed under the Apache-2.0 License.

import 'package:violet/script/parser.dart';
import 'package:violet/script/script_model.dart';

class ScriptParser extends ShiftReduceParser {
  static const Map<String, int> _symbolTable = {
    "S'": 0,
    "script": 1,
    "line": 2,
    "stmt": 3,
    "block_entry": 4,
    "block": 5,
    "consts": 6,
    "index": 7,
    "variable": 8,
    "argument": 9,
    "function": 10,
    "runnable": 11,
    "=": 12,
    "[": 13,
    "]": 14,
    "number": 15,
    "string": 16,
    "name": 17,
    ",": 18,
    "(": 19,
    ")": 20,
    "loop": 21,
    "to": 22,
    "foreach": 23,
    ":": 24,
    "if": 25,
    "else": 26,
    "\$": 27,
  };

  static const List<List<int>> _gotoTable = [
    [
      0,
      1,
      4,
      5,
      2,
      0,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      3,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      54
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -1
    ],
    [
      0,
      0,
      18,
      5,
      0,
      17,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      0,
      -9,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -7,
      -7,
      -7,
      -7,
      0,
      0,
      0,
      -7,
      0,
      -7,
      0,
      -7,
      -7,
      -7
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -2,
      -2,
      -2,
      -2,
      0,
      0,
      0,
      -2,
      0,
      -2,
      0,
      -2,
      -2,
      -2
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -15,
      -15,
      -3,
      -3,
      -3,
      -3,
      0,
      0,
      0,
      -3,
      0,
      -3,
      0,
      -3,
      -3,
      -3
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      19,
      20,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -5,
      -5,
      -5,
      -5,
      0,
      0,
      0,
      -5,
      0,
      -5,
      0,
      -5,
      -5,
      -5
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -14,
      -14,
      -14,
      -14,
      -14,
      -14,
      -14,
      21,
      -14,
      -14,
      -14,
      -14,
      0,
      -14,
      -14,
      -14
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -12,
      -12,
      -12,
      -12,
      -12,
      -12,
      -12,
      0,
      -12,
      -12,
      -12,
      -12,
      0,
      -12,
      -12,
      -12
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      22,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      23,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      24,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -16,
      -16,
      -16,
      -16,
      -16,
      -16,
      -16,
      0,
      -16,
      -16,
      -16,
      -16,
      0,
      -16,
      -16,
      -16
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -10,
      -10,
      -10,
      -10,
      -10,
      -10,
      -10,
      0,
      -10,
      -10,
      -10,
      -10,
      0,
      -10,
      -10,
      -10
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -11,
      -11,
      -11,
      -11,
      -11,
      -11,
      -11,
      0,
      -11,
      -11,
      -11,
      -11,
      0,
      -11,
      -11,
      -11
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      25,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      18,
      5,
      0,
      26,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      0,
      -9,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      27,
      10,
      0,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      0,
      29,
      0,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      32,
      10,
      31,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      30,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      33,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      34,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      35,
      10,
      0,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -6,
      -6,
      -6,
      -6,
      0,
      0,
      0,
      -6,
      0,
      -6,
      0,
      -6,
      -6,
      -6
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -8,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      20,
      -4,
      -4,
      -4,
      -4,
      0,
      0,
      0,
      -4,
      0,
      -4,
      0,
      -4,
      -4,
      -4
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -15,
      -15,
      -15,
      -15,
      -15,
      -15,
      0,
      -15,
      -15,
      -15,
      -15,
      0,
      -15,
      -15,
      -15
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      36,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -19,
      -19,
      -19,
      -19,
      -19,
      -19,
      -19,
      0,
      -19,
      -19,
      -19,
      -19,
      0,
      -19,
      -19,
      -19
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      37,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      20,
      0,
      0,
      0,
      0,
      38,
      0,
      -17,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      39,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      40,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      20,
      0,
      0,
      0,
      0,
      0,
      0,
      41,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -13,
      -13,
      -13,
      -13,
      -13,
      -13,
      -13,
      0,
      -13,
      -13,
      -13,
      -13,
      0,
      -13,
      -13,
      -13
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -20,
      -20,
      -20,
      -20,
      -20,
      -20,
      -20,
      0,
      -20,
      -20,
      -20,
      -20,
      0,
      -20,
      -20,
      -20
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      32,
      10,
      42,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      43,
      10,
      0,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      44,
      10,
      0,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      4,
      5,
      45,
      0,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      3,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -18,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      20,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      46,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      20,
      0,
      0,
      0,
      0,
      0,
      0,
      47,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -23,
      -23,
      -23,
      -23,
      0,
      0,
      0,
      -23,
      0,
      -23,
      0,
      -23,
      48,
      -23
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      14,
      49,
      10,
      0,
      28,
      0,
      0,
      0,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      4,
      5,
      50,
      0,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      3,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      4,
      5,
      51,
      0,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      3,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      20,
      0,
      0,
      0,
      0,
      0,
      0,
      52,
      0,
      0,
      0,
      0,
      0,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -22,
      -22,
      -22,
      -22,
      0,
      0,
      0,
      -22,
      0,
      -22,
      0,
      -22,
      -22,
      -22
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -24,
      -24,
      -24,
      -24,
      0,
      0,
      0,
      -24,
      0,
      -24,
      0,
      -24,
      -24,
      -24
    ],
    [
      0,
      0,
      4,
      5,
      53,
      0,
      14,
      7,
      10,
      0,
      6,
      8,
      0,
      3,
      0,
      15,
      16,
      9,
      0,
      0,
      0,
      11,
      0,
      12,
      0,
      13,
      0,
      0
    ],
    [
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      0,
      -21,
      -21,
      -21,
      -21,
      0,
      0,
      0,
      -21,
      0,
      -21,
      0,
      -21,
      -21,
      -21
    ],
  ];

  static const List<int> _production = [
    1,
    1,
    1,
    1,
    3,
    1,
    3,
    1,
    2,
    0,
    1,
    1,
    1,
    4,
    1,
    1,
    1,
    1,
    3,
    3,
    4,
    9,
    7,
    5,
    7
  ];

  static const List<int> _groupTable = [
    0,
    1,
    2,
    3,
    3,
    3,
    4,
    4,
    5,
    5,
    6,
    6,
    7,
    7,
    8,
    8,
    8,
    9,
    9,
    10,
    10,
    11,
    11,
    11,
    11
  ];

  static const int _accept = 54;

  ScriptParser()
      : super(
          symbolTable: _symbolTable,
          table: _gotoTable,
          production: _production,
          groupTable: _groupTable,
          accept: _accept,
          actions: PActionDescription.actions,
        );
}
